from cprints.color import Color
from cprints.style import Style

PREFIX = "\033["
SUFFIX = "\033[0m"


class ColorPrintConfig:
    # 字体颜色
    black   = Color("black")
    red     = Color("red")
    green   = Color("green")
    yellow  = Color("yellow")
    blue    = Color("blue")
    magenta = Color("magenta")
    cyan    = Color("cyan")
    white   = Color("white")

    # 字体样式
    highlight  = Style("highlight")
    underline  = Style("underline")
    flash      = Style("flash")
    reverse    = Style("reverse")
    no_visible = Style("no_visible")

    # 默认字体颜色和样式
    default_color = Color("default")
    default_style = Style("default")

    set_style      = Style()._set_style
    set_foreground = Color()._set_foreground
    set_background = Color()._set_background



from cprints.color import Color as colors
from cprints.style import Style as styles
from cprints.main  import ColorPrint


cp = ColorPrint()

# 彩色打印
cprint        = cp.cprint
print_black   = cp.print_black
print_red     = cp.print_red
print_green   = cp.print_green
print_yellow  = cp.print_yellow
print_blue    = cp.print_blue
print_magenta = cp.print_magenta
print_cyan    = cp.print_cyan
print_white   = cp.print_white

# 着色文本
colored         = cp.colored
colored_black   = cp.colored_black
colored_red     = cp.colored_red
colored_green   = cp.colored_green
colored_yellow  = cp.colored_yellow
colored_blue    = cp.colored_blue
colored_magenta = cp.colored_magenta
colored_cyan    = cp.colored_cyan
colored_white   = cp.colored_white

# 进度条
progress_bar    = cp.progress_bar

__all__ = ["cp", "colors", "styles"]



class Color:

    default = {"fore": 39, "back": 49, "explain": "default", "desc": "默认"}
    black   = {"fore": 30, "back": 40, "explain": "black",   "desc": "黑色"}
    red     = {"fore": 31, "back": 41, "explain": "red",     "desc": "红色"}
    green   = {"fore": 32, "back": 42, "explain": "green",   "desc": "绿色"}
    yellow  = {"fore": 33, "back": 43, "explain": "yellow",  "desc": "黄色"}
    blue    = {"fore": 34, "back": 44, "explain": "blue",    "desc": "蓝色"}
    magenta = {"fore": 35, "back": 45, "explain": "magenta", "desc": "洋红"}
    cyan    = {"fore": 36, "back": 46, "explain": "cyan",    "desc": "青色"}
    white   = {"fore": 37, "back": 47, "explain": "white",   "desc": "白色"}

    def __init__(self, color=None):
        self._color      = color
        self._all_colors = dict()
        for name in dir(self):
            if not name.startswith("_"):
                self._all_colors[name] = getattr(self, name)

    def __get__(self, instance, owner):
        return dict(fore=self._set_foreground(self._color),
                    back=self._set_background(self._color))

    def _set_foreground(self, color, _key="fore"):
        # color: dict -> 查找 color[_key]
        if isinstance(color, dict) \
            and color.__contains__(_key):

            for _color in self._all_colors.values():
                if str(color[_key]) == str(_color[_key]):
                    color = _color[_key]
                    break
            else:
                color = self.default[_key]

        # color: str -> 对比各项数据
        elif isinstance(color, (str, int)):
            color = str(color).lower()
            for _color in self._all_colors.values():
                for value in _color.values():
                    if color == str(value):
                        color = _color[_key]
                        break
                else:
                    continue
                break
            else:
                color = self.default[_key]
        # 其他
        else:
            color = self.default[_key]
        return str(color)

    def _set_background(self, color, _key="back"):
        # color: dict -> 查找 color[_key]
        if isinstance(color, dict) \
            and color.__contains__(_key):

            for _color in self._all_colors.values():
                if str(color[_key]) == str(_color[_key]):
                    color = _color[_key]
                    break
            else:
                color = self.default[_key]

        # color: str -> 对比各项数据
        elif isinstance(color, (str, int)):
            color = str(color).lower()
            for _color in self._all_colors.values():
                for value in _color.values():
                    if color == str(value):
                        color = _color[_key]
                        break
                else:
                    continue
                break
            else:
                color = self.default[_key]
        # 其他
        else:
            color = self.default[_key]
        return str(color)



class Style:

    default    = {"style": 0,  "explain": "default",    "desc": "默认"}
    highlight  = {"style": 1,  "explain": "highlight",  "desc": "高亮"}
    underline  = {"style": 4,  "explain": "underline",  "desc": "下划线"}
    flash      = {"style": 5,  "explain": "flash",      "desc": "闪烁"}
    reverse    = {"style": 7,  "explain": "reverse",    "desc": "反显"}
    no_visible = {"style": 8,  "explain": "no_visible", "desc": "不可见"}

    def __init__(self, style=None):
        self._style      = style
        self._all_styles = dict()
        for name in dir(self):
            if not name.startswith("_"):
                self._all_styles[name] = getattr(self, name)

    def __get__(self, instance, owner):
        return self._set_style(self._style)

    def _set_style(self, style, _key="style"):
        # style: dict -> 查找 style[_key]
        if isinstance(style, dict) \
            and style.__contains__(_key):

            for _style in self._all_styles.values():
                if str(style[_key]) == str(_style[_key]):
                    style = _style[_key]
                    break
            else:
                style = self.default[_key]

        # style: str -> 对比各项数据
        elif isinstance(style, (str, int)):
            style = str(style).lower()
            for _style in self._all_styles.values():
                for value in _style.values():
                    if style == str(value):
                        style = _style[_key]
                        break
                else:
                    continue
                break
            else:
                style = self.default[_key]

        # style: list
        elif isinstance(style, (list, tuple)):
            style = set([self._set_style(i) for i in style])
            style = ";".join(list(style))

        # 其他
        else:
            style = self.default[_key]

        return str(style)



import math
from cprints.config import (
    PREFIX, SUFFIX, ColorPrintConfig)


class BasePrint(ColorPrintConfig):

    def colored(self, text, /, *args, fg=None, bg=None, style=None, suffix=True):
        """\
        :params text   : 着色文本
        :params *args  : 着色可变参数
        :params fg     : 前景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params bg     : 背景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params style  : 文本样式(default, highlight, underline, flash, reverse)
        :params suffix : 恢复默认打印方式

        explain:
            from cprints import cp
            text = cp.colored("Hello World", fg=cp.red, bg=cp.black, style=cp.underline)
            print(text)
        """
        fg     = self.set_foreground(fg)
        bg     = self.set_background(bg)
        style  = self.set_style(style)
        suffix = SUFFIX if bool(suffix) else ""
        args = " ".join([str(i) for i in args])
        if args:
            args = " " + args
        return f"{PREFIX}{style};{fg};{bg}m{text}{args}{suffix}"

    def cprint(self, text, /, *args, fg=None, bg=None, style=None, suffix=True, **kwargs):
        """\
        :params text     : 着色文本
        :params *args    : 着色可变参数
        :params fg       : 前景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params bg       : 背景色(black, red, green, yellow, blue, magenta, cyan, white)
        :params style    : 文本样式(default, highlight, underline, flash, reverse)
        :params suffix   : 恢复默认打印方式
        :params **kwargs : print的关键字参数

        explain:
            from cprints import cp
            cp.cprint("Hello World", fg=cp.red, bg=cp.black, style=cp.underline)
        """
        fg     = self.set_foreground(fg)
        bg     = self.set_background(bg)
        style  = self.set_style(style)
        suffix = SUFFIX if bool(suffix) else ""
        args = " ".join([str(i) for i in args])
        if args:
            args = " " + args
        print(f"{PREFIX}{style};{fg};{bg}m{text}{args}{suffix}", **kwargs)

    def progress_bar(self, now: int, total: int, lenght=36, desc="", bar=b"\xe2\x94\x81", pad=""):
        """\
        :params now    : 当前进度值
        :params total  : 总进度值
        :params lenght : 进度条长度
        :params desc   : 进度条介绍
        :params bar    : 进度条 进度填充
        :params pad    : 进度条 其余填充

        explain:
            from cprints import cp
            maximum = 100
            for i in range(1, maximum+1):
                cp.progress_bar(now=i, total=maximum)
        """
        now = total if now >= total else now
        bar = bar.decode() if isinstance(bar, bytes) else str(bar)
        # 百分比
        percent = now / total
        # 进度条总长度
        total_bar = len(self.colored(lenght * bar))
        foreground = self.red if now < total else self.green
        # 进度条
        progress_bar = self.colored(
            math.floor(percent * lenght) * bar, fg=foreground)
        # 需要打印的百分比
        percent = self.colored(f"{percent:.2%}", style=self.underline)
        end = "\n" if now >= total else ""
        print(f"\r<{desc}{progress_bar:{pad}<{total_bar}}> {percent} [{now}/{total}]", end=end, flush=True)


class Colored(BasePrint):

    def colored_black(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.black, bg=bg, style=style, suffix=suffix)

    def colored_red(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.red, bg=bg, style=style, suffix=suffix)

    def colored_green(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.green, bg=bg, style=style, suffix=suffix)

    def colored_yellow(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.yellow, bg=bg, style=style, suffix=suffix)

    def colored_blue(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.blue, bg=bg, style=style, suffix=suffix)

    def colored_magenta(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.magenta, bg=bg, style=style, suffix=suffix)

    def colored_cyan(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.cyan, bg=bg, style=style, suffix=suffix)

    def colored_white(self, text, /, *args, bg=None, style=None, suffix=True):
        return self.colored(text, *args, fg=self.white, bg=bg, style=style, suffix=suffix)


class ColorPrint(Colored):

    def print_black(self, text ,/, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.black, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_red(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.red, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_green(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.green, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_yellow(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.yellow, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_blue(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.blue, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_magenta(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.magenta, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_cyan(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.cyan, bg=bg, style=style, suffix=suffix, **kwargs)

    def print_white(self, text, /, *args, bg=None, style=None, suffix=True, **kwargs):
        self.cprint(text, *args, fg=self.white, bg=bg, style=style, suffix=suffix, **kwargs)
